% Represent the samples with the sparse coefficients. SIFT feature is used
% here
% input:
%     X: the warp images set X
%     tmpl: the classifier parameters
%     opt: golbal parameters
% output: 
%     A: the learned coefficient set

function Fea = CropFea(X, opt)

% the size of image patch is fixed for each particle
im_h = opt.tmplsize(1);
im_w = opt.tmplsize(2);

gridSpacing = opt.gridSpacing;
patchSize = opt.patchSize;

% make grid sampling SIFT descriptors
remX = mod(im_w-patchSize,gridSpacing);
offsetX = floor(remX/2)+1;
remY = mod(im_h-patchSize,gridSpacing);
offsetY = floor(remY/2)+1;

[gridX,gridY] = meshgrid(offsetX:gridSpacing:im_w-patchSize+1, offsetY:gridSpacing:im_h-patchSize+1);

dim = length(gridX(:));
num_samples = size(X,3);

% change to patch type
%=====================================================================
% first step - extract SIFT feature fro all samples
% record the SIFT features from all particles and arrange them in a matrix.

SIFT_fea = zeros(opt.patchSize*opt.patchSize, dim*num_samples);

for i=1:num_samples % the number of positive and negtive samples  
    tmp = X(:,:,i);
    I = tmp;

    % raw gray image
    B_all = zeros(opt.patchSize*opt.patchSize, dim);
    for j = 1:dim
        tmp = I((gridY(j)):(gridY(j))+patchSize-1, (gridX(j)):(gridX(j))+patchSize-1);
        tmp = tmp(:);
        B_all(:,j) = tmp;
    end
    SIFT_fea(:,(i-1)*dim+1:i*dim) = B_all;    
end
Fea = SIFT_fea;
