% Represent the samples with the sparse coefficients. SIFT feature is used
% here
% input:
%     X: the warp images set X
%     tmpl: the classifier parameters
%     opt: golbal parameters
% output: 
%     A: the learned coefficient set

function A = SparseRepresentImGPU(X, opt, D)

% the size of image patch is fixed for each particle
im_h = opt.tmplsize(1);
im_w = opt.tmplsize(2);

gridSpacing = opt.gridSpacing;
patchSize = opt.patchSize;

% make grid sampling
remX = mod(im_w-patchSize,gridSpacing);
offsetX = floor(remX/2)+1;
remY = mod(im_h-patchSize,gridSpacing);
offsetY = floor(remY/2)+1;

[gridX,gridY] = meshgrid(offsetX:gridSpacing:im_w-patchSize+1, offsetY:gridSpacing:im_h-patchSize+1);

dim = length(gridX(:));
num_samples = size(X,3);

% change to patch type
dSize = size(D, 2);  
%=====================================================================
% store all the image patches
All_fea = zeros(opt.patchSize*opt.patchSize, dim*num_samples);
% A = [];


if opt.ispool == 1
    % for pooling
    tmp = opt.pyramid;
    tmp = sum(tmp.^2);
    A = zeros(dSize*tmp, num_samples);
else
    % for concatenation
    A = zeros(dSize*dim, num_samples); % for concatenation
end
    
    

feaSet.x = gridX(:) + patchSize/2 - 0.5;
feaSet.y = gridY(:) + patchSize/2 - 0.5;

for i=1:num_samples % the number of positive and negtive samples  
    I = X(:,:,i);
    % raw gray image
    B_all = zeros(opt.patchSize*opt.patchSize, dim);    
    for j = 1:dim
        tmp = I((gridY(j)):(gridY(j))+patchSize-1, (gridX(j)):(gridX(j))+patchSize-1);
        B_all(:,j) = tmp(:);        
    end
    All_fea(:,(i-1)*dim+1:i*dim) = B_all;      
end
%======================================================================
%% sparse coding with GPU
% b = -D' * All_fea; 
% % tic;
% B = D' * D + opt.lambda*eye(size(D,2));
% % toc;
% %------------------------------------------------
% d_x0 = zeros(size(D,2),size(All_fea, 2),GPUsingle);
% d_b = GPUsingle(b);
% d_B = GPUsingle(B);
% % tic;
% % x2 = gpu_Coordinate_Descent(d_B,d_b,d_x0,0.15,0.4,50);
% % Xout = single(x2);
% % t4 = toc;
% d_alpha = ones(1,size(All_fea,2),GPUsingle);
% d_idx = d_alpha;
% % d_lambda = 0.15;
% d_rho = 0.5;
% d_c = 1e-6;
% d_lsiter = 5;
% d_maxiter = 50;
% tmp = gpu_ls_Coordinate_Descent(d_B,d_b,d_x0,d_alpha,d_idx,...
%     0.15,d_rho,d_c,d_lsiter,d_maxiter);
% all_codes = single(tmp);

%% sparse coding without CPU
all_codes = l1ls_featuresign (D, All_fea, opt.gamma); %

%================================================================
feaSet.width = im_w;
feaSet.height = im_h;
% 
for i=1:num_samples % the number of positive and negtive samples    
    feaSet.feaArr = All_fea(:,1+(i-1)*dim:i*dim); %select the corresponding sift feature set of the ith particle

    tmp = all_codes(:,1+(i-1)*dim:i*dim);
    if opt.ispool == 1
        % pooling
        beta = Pooling(feaSet, dSize, tmp, opt.pyramid);
        A(:,i) = beta;
    else
        A(:,i) = tmp(:);
    end
end

