%% Discriminative sparse representation + local patch model
%% Copyright (C)Qing Wang.
%% All rights reserved.
clear all;
close all;
clear;
clc;
addpath(genpath('fast_sc\'));
addpath(genpath('liblinear-1.51\'));

warning off all;

% savepath;
iptsetpref('ImshowBorder', 'tight'); %drop the display boundary


%dir
prefix = 'Sequences\car\';

%location of the tracking result
prefix_result = 'result\';
% start frame
fstart = 1;  

% first frame
fname = sprintf('%s%04d.jpeg',prefix,fstart);  %

%read the sequence
im = imread(fname);
f_frame = sprintf('%d',1);
fprintf('tracking frame: # %s\n', f_frame); 

figure(1);
imshow(im);

%% locate the target object manually
%get 4 vertices of object
% fprintf('Please click 4 corners of the object to track:');
% [xbox,ybox] = ginput(4);  %user inputs corners of bounding box
% %get object bounding box
% xbox = round(xbox);   %get integer value
% ybox = round(ybox);
% left = min(xbox);    
% right = max(xbox); 
% top = min(ybox);     
% bottom = max(ybox);
% objBox = [left,right,top,bottom];      
% objCenterx = round((left+right)/2);
% objCentery = round((top+bottom)/2);
% width = right-left;
% height = bottom-top;
% %the intialized parameters
% p = [objCenterx, objCentery, width, height, 0];  %x, y, w, h, rotation, angle

%% locate the target object 
% location x, location y, width, height, rotation
p = [108,103,74,58,0];  %car

%% parameter setting

opt = struct('numsample', 600, 'condenssig',1, 'posNum',30, 'negNum',30, 'graError',0.1, 'maxGraIteNum',15, ...
    'tmplsize', [32, 32], 'batchsize',1,'r',2, 'tmplNum', 2, 'nrml_threshold',1, 'gridSpacing',8, 'c',10, ...
    'patchSize',16, 'maxImSize',32, 'gamma', 0.15, 'ispool', 0, ...
    'lambda', 0.0001, 'pyramid', [1, 2], 'conf', 0.8, ...
    'bk_cons_mat', [1,1,0,0; 0,0,1,1; -0.8, 0.8,0,0; 0,0,-0.8,0.8]); %'pyramid', [0.25, 0.5, 1]



%% predicting standard covariance matrix (diagonal)
% center_x, center_y, width, height/width
opt.affsig = [4, 4, 0.01, 0,0,0]; % car

%% diagonal covariance matrix for positive collection
opt.posSig = [4,4,.01,.02,.002,.001];

frame = rgb2gray(im);
% frame = im;
frame =  double(frame)/256;
param0 = [p(1), p(2), p(3)/opt.tmplsize(2), p(5), p(4)/p(3), 0];
param0 = affparam2mat(param0);
param.est = param0';

%%
% draw initial track window
figure(1);
drawbox(32,32, param.est, 'Color','r', 'LineWidth',2.5);

Current_title = sprintf('# %d',fstart);
text(5, 18, Current_title, 'Color','y', 'FontWeight','bold', 'FontSize',12);
wname = sprintf('%s%04d.jpeg', prefix_result, fstart); %?????????
imwrite(frame2im(getframe(gcf)),wname);
% pause;
step = 1;
hwin=gcf;
frameIndex = fstart;
Current_title = sprintf('# %d',frameIndex);
set(hwin, 'Name', Current_title);
wimgs = []; 

%% Initialize the claffifier in the first tracking frame
% select a lot of positive samples
[posSample, posPosition] = SelectPos(frame, opt.tmplsize, param.est, opt.posNum, opt.posSig);
% display the positive samples
mri = zeros(opt.tmplsize(1),opt.tmplsize(2),1,opt.posNum);
mri(:,:,1,:)=posSample(:,:,:);
figure(2);
montage(mri);
[negSample, position] = SelectNeg(frame, param0, opt);
mri = zeros(opt.tmplsize(1),opt.tmplsize(2),1,opt.negNum);
mri(:,:,1,:)=negSample(:,:,:);
figure(3);
montage(mri);
X = cat(3, posSample, negSample);

%-----------------------------------------------------
% use the observed positive and negative patches and trivial templates to
% construct the over-complete dictionary
Fea = CropFea(posSample(:,:,opt.posNum), opt); % only select from one sample
tmp2 = eye(opt.patchSize*opt.patchSize);
tmpl.D = cat(2, Fea, tmp2, -tmp2);

% ================================================================
% Extract the descriptor based on SIFT, and coding them by the learned
% dictionary
% tic;
% dbstop if error
F = SparseRepresentImGPU(X, opt, tmpl.D);
% ================================================================


%% Train the logistic classifier
% X data
% L label
L1 = ones(1,opt.posNum);
L2 = -ones(1,opt.negNum);
L = [L1 L2];
dim_c = size(F,1);
% model = train( L', sparse(F), '-s 0 -B 1 -c 0.1', 'col' );  %the character after-c is the regularization parameter for linear SVM 
% tic;
%--------------------------------------------------------------------------
% For L2-regularized logistic regression (-s 0), we solve
% min_w w^Tw/2 + opt.c \sum log(1 + exp(-y_i w^Tx_i))
model = train( L', sparse(F), '-s 0 -B 0 -c 10', 'col' );  %2/lamda = 10,lamda = 0.2

w = model.w(1:dim_c); 
w = w';
b = model.w(dim_c+1); 
% toc;

%store the initial model
% the classifier used in our algorithm; used as a fixed detector ;-)
tmpl.w = w;
tmpl.b = b;
w = [tmpl.w;tmpl.b]; %augmented
% define an online detector
tmpl.wo = tmpl.w;
tmpl.bo = tmpl.b;

tmplo = tmpl;

% ========================================================================
% test the training data to compare the classification results and the
% ground truth for the training samples
F1 = F(:,1:opt.posNum);
P1 = 1 ./ (1 + exp(-(tmpl.w'*F1+tmpl.b)));
% for the negtive samples:
F0 = F(:,opt.posNum+1:opt.posNum+opt.negNum);
P0 = 1 ./ (1 + exp(-(tmpl.w'*F0+tmpl.b)));
% 
%Visualize the classification results
figure(4);
index1 = 1:opt.posNum;
index2 = 1:opt.negNum;
plot(index1, P1, '-gs');
hold on;
plot(P0, '--r.');
legend('postive', 'negtive', 2);


%%
% pause;
Dir = dir(prefix);
fNum = size(Dir,1)-3; %

paramRes = zeros(6, fNum);    % store the tracking result
paramRes(:,1) = param.est;
paramRes_predict = zeros(4, fNum); 
paramRes_predict(:,1) = [0 0 0 0]';

%% for classifier update
obserSet = posSample(:,:,opt.posNum);
nu = 0.2;  
tmpl.D0 = tmpl.D;
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                             %The main tracking algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%tracking from the 2th frame
for frameIndex = fstart+1:step:fNum
%     tic;
    frameIndex1 = mod(frameIndex,opt.batchsize);  %
    % read the next frame
    fname = sprintf('%s%04d.jpeg',prefix,frameIndex);  %
    wname = sprintf('%s%04d.jpeg', prefix_result, frameIndex); %
    im = imread(fname);
    
    %display
    figure(1);
    imshow(im);   
    f_frame = sprintf('%d',frameIndex);
    fprintf('tracking frame: the %sth frame\n', f_frame); 
      
    frame = rgb2gray(im); 
%     frame = im;  %for freeman
    frame = double(frame)/256;
    
    % first use the latest template (dictionary) for tracking result
    tmp = tmpl.D; % store the latest dictionary
    param = estwarp_condens(frame, tmpl, param, opt);
    % then use the initial dictionary to refine the tracking result
    param = estwarp_condens(frame, tmplo, param, opt); % final tracking result
    
    % record the tracking result
    paramRes(:,frameIndex) = param.est;
    paramRes_predict(:,frameIndex) = param.bkBox;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%Update the classifier%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       obserSet = cat(3, obserSet, param.wimg);    
    if (size(obserSet,3) == opt.batchsize)         
        posSample = obserSet;
        % Collect some negative samples around the positive one
        [negSample, position] = SelectNeg(frame, param.est, opt);
        % all the samples
        X = cat(3, posSample, negSample);
        %% update the over-complete dictionary       
        Fea = CropFea(posSample, opt);
        % update the dictionary in each frame or not
        tmpl.D = cat(2, Fea, tmp2, -tmp2);

       %% update the classifier
        % represent the samples by patch based sparse coding
        train_data = SparseRepresentImGPU(X, opt, tmpl.D);
%         train_data = SparseRepresentIm(X, opt, tmpl.D);
        % update the dictionary
        dim_c = size(train_data,1);
        % update the classifier
        % label
        L_pos = ones(1,opt.batchsize);
        L_neg = -ones(1,opt.negNum);
        L = [L_pos L_neg];
        %--------------------------------------------------------------
        % directly learn a new SVM classifier
        model = train( L', sparse(train_data), '-s 0 -B 1 -c 10', 'col' );  
        w = model.w(1:dim_c);
        w = w';
        b = model.w(dim_c+1);
        tmpl.w  = w;
        tmpl.b = b;        
        obserSet = [];
    end
     
%   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%display the tracking result%%%%%%%%%%%%%%%%%%%%%%%%
    %% draw result    
    figure(1);
    drawbox(opt.tmplsize(2),opt.tmplsize(2), param.est, 'Color','r', 'LineWidth',2.5);
    Current_title = sprintf('# %d',frameIndex);
    set(hwin, 'Name', Current_title);
    text(5, 18, Current_title, 'Color','y', 'FontWeight','bold', 'FontSize',12);    
    imwrite(frame2im(getframe(gcf)),wname);    
end
paraSaveresult = sprintf('%s%s.mat','plot_result\','result'); 
paraSavePre = sprintf('%s%s.mat','plot_result\','pre'); 
save(paraSaveresult, 'paramRes','opt');
save(paraSavePre, 'paramRes_predict','opt');


